<?php
/*
Plugin Name: Fun with in-context comments
Plugin URI: [insert the plugin uri here]
Description: Adds context information to user comments
Author: Andrew Rickmann
Version: 0.1.1
Author URI: http://www.wp-fun.co.uk
Generated At: www.wp-fun.co.uk;
*/ 

/*  Copyright 2008  Andrew Rickmann  (email : mail@arickmann.co.uk)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

if (!class_exists('fw_in_context_comments')) {
    class fw_in_context_comments	{
		
		/**
		* @var string   The name the options are saved under in the database.
		*/
		var $adminOptionsName = "fw_in_context_comments_options";
		
		
		/**
		* @var string   The name of the database table used by the plugin
		*/	
		var $db_table_name = '';

		
		/**
		* PHP 4 Compatible Constructor
		*/
		function fw_in_context_comments(){$this->__construct();}
		
		/**
		* PHP 5 Constructor
		*/		
		function __construct(){
			global $wpdb;

			add_action('admin_head' , array( &$this , 'add_meta_boxes' ));
			add_action('admin_head' , array( &$this , 'add_meta_boxes' ));
			add_action('wp_insert_post' , array( &$this , 'save_context_info' ) , 10, 2); 
			add_action('comment_post' , array( &$this , 'save_comment_info' ) , 10, 1); 
			add_action('delete_comment' , array( &$this , 'delete_comment_info' ) , 10, 1);
			add_action("admin_menu", array(&$this,"add_admin_pages"));
			register_activation_hook(__FILE__,array(&$this,"install_on_activation"));
			
			$this->adminOptions = $this->getAdminOptions();
			
			//set the appropriate filter value
			if ( $this->adminOptions['settings']['results_position'] == 'above' ) { $comment_text_priority = 1000; } else { $comment_text_priority = 1; }
			
			add_filter('comment_text', array(&$this, 'add_context_to_coment_text'), $comment_text_priority);
	
			$fun_with_in_context_comments_locale = get_locale();
			$fun_with_in_context_comments_mofile = dirname(__FILE__) . "/languages/fun_with_in_context_comments-".$fun_with_in_context_comments_locale.".mo";
			load_textdomain("fun_with_in_context_comments", $fun_with_in_context_comments_mofile);
		
			$this->db_table_name = $wpdb->prefix . "comment_contexts";

		}
		
		/**
		 * add_context_to_comment_text - adds the commeters choices to the comment text
		 * 
		 * @return String The comment text
		 * @param $content String the comment text
		 */
		function add_context_to_coment_text($content){
			global $comment, $wpdb;
			
			//make sure we are in admin
			if ( is_admin() ){
				
				//get all values for this comment
				$values = $wpdb->get_results( sprintf("SELECT * FROM `%s` WHERE `comment_id` = %d" , $this->db_table_name , $comment->comment_ID)  );
			
				if ( count($values) > 0 ){
					$output_string = '';
					foreach ( $values as $value ){
						if ( $output_string != '' ){
							$output_string .= '<br />';
						}
					$output_string .= $value->comment_field . ' : ' . $value->comment_value;
					}
				return $content . '<h4 style="margin-bottom:0">' . __('Context' , "fun_with_in_context_comments" ) . '</h4>' . '<p style="margin-top:0">' . $output_string . '</p>';
				}			
			} else {
				
				//get the stuff to add it in
				$context = $this->output_context_results( $this->adminOptions['settings']['before'] ,  $this->adminOptions['settings']['separator'] , $this->adminOptions['settings']['between'], $this->adminOptions['settings']['none']);
				
				if ( $this->adminOptions['settings']['results_position'] == 'above' ){
					return str_replace( '%content%' , $context  , $this->adminOptions['settings']['template'] ) . $content;
				} else {
					return $content . str_replace( '%content%' , $content  , $this->adminOptions['settings']['template'] );					
				}
				
				
			}
			
			//if not already returned
			return $content;
		}
		
		/**
		 * delete_comment_info
		 * 
		 * Deletes any context information for the specified comment.
		 *  
		 * @param $comment_id Integer The ID of the comment that is being deleted
		 */
		function delete_comment_info( $comment_id ){
			global $wpdb;
			//the delete query
			$delete_query = sprintf( "DELETE FROM `%s` WHERE `comment_id` = %d" , $this->db_table_name , $comment_id );
			
			//do the deletion
			$wpdb->query( $delete_query );
						
		}
		
		/**
		 * save_comment_info
		 * 
		 * Saves the choices the user enters.
		 * 
		 * @param $comment_id Integer The ID of the comment that the user has just entered.
		 */
		function save_comment_info( $comment_id ){
			global $wpdb;
			
			if ( !wp_verify_nonce( $_POST['fun_with_in_context_comments_record_comment_context'], plugin_basename(__FILE__) )) {
			  return;
			}
			
			//use the comment ID to get the post ID
			$comment = get_comment($comment_id);
			$post_id = $comment->comment_post_ID;
			$update_sql = "UPDATE `%s` SET `comment_field` = '%s', `comment_value` = '%s' WHERE `comment_id` = %d AND `comment_field` = '%s'";
			$new_row_sql = "INSERT INTO `%s` SET `comment_id` = %d , `comment_field` = '%s', `comment_value` = '%s'";
			
			//get global and local fields
			$global_fields = $this->adminOptions['global_contexts'];
			$local_fields = get_post_meta( $post_id , 'fun_with_in_context_comments' , true);
			
			//do the global fields
			foreach ( $global_fields as $index => $global_field ) {
				//if the global field has been deleted then skip
				if ( $global_field == 'deleted' ){ continue; }
				if ( isset( $_POST['fwicc_global_' . $index ] ) && is_numeric( $_POST['fwicc_global_' . $index ]) && $_POST['fwicc_global_' . $index ] < count( $global_field['values'] ) ){
					
					$field_key = $global_field['title'];
					$context_value = $global_field['values'][$_POST['fwicc_global_' . $index ]];

					$update_query = sprintf( $update_sql , $this->db_table_name , $field_key , $context_value , $comment_id , $field_key );
					$new_row_query = sprintf( $new_row_sql , $this->db_table_name , $comment_id , $field_key , $context_value );
						
					if ( !$wpdb->query( $update_query ) ){
						$wpdb->query( $new_row_query );
					}
					
				}	
			}
			
			
			//Do the local fields
			if ( $local_fields !== false ) {
				foreach ( $local_fields as $index => $local_field ) {
					
					if ( isset( $_POST['fwicc_local_' . $index ] ) && is_numeric( $_POST['fwicc_local_' . $index ]) && $_POST['fwicc_local_' . $index ] < count( $local_field['values'] ) ){
						
						$field_key = $local_field['title'];
						$context_value = $local_field['values'][$_POST['fwicc_local_' . $index ]];
	
						$update_query = sprintf( $update_sql , $this->db_table_name , $field_key , $context_value , $comment_id , $field_key );
						$new_row_query = sprintf( $new_row_sql , $this->db_table_name , $comment_id , $field_key , $context_value );
							
						if ( !$wpdb->query( $update_query ) ){
							$wpdb->query( $new_row_query );
						}
						
					}	
				}
			} 
					
			
		}

		/**
		 * save_context_info
		 * 
		 * Saves the data entered on the post screen, that selects which options to present the user with.
		 * 
		 * @return 
		 * @param $post_id integer the ID of the post that the coments belong to.
		 * @param $post Object[optional] The full post object.
		 */
		function save_context_info( $post_id , $post = null){
			
			// verify this came from the our screen and with proper authorization,
			// because save_post can be triggered at other times
			
			if ( !wp_verify_nonce( $_POST['fun_with_in_context_comments_add_context_fields'], plugin_basename(__FILE__) )) {
			  return $post_id;
			}
			
			if ( 'page' == $_POST['post_type'] ) {
			  if ( !current_user_can( 'edit_pages', $post_id ))
			  	return $post_id;
			} else {
			  if ( !current_user_can( 'edit_posts', $post_id ))
				return $post_id;
			}
			
			// OK, we're authenticated: we need to find and save the data
			
			//start with the globals
			if ( isset( $_POST['global_contexts']) ){
				if ( update_post_meta($post_id, 'fun_with_in_context_comments_globals', $_POST['global_contexts'] ) == false ){
					add_post_meta($post_id, 'fun_with_in_context_comments_globals', $_POST['global_contexts'] );
				}
			}
					
			//now do the additionals
			
			$comment_context_array = array();
			$context_field_counter = 0;
				
			foreach( $_POST['comment_context_field_titles'] as $title ){
			
			//make sure all the fields have been completed
			if ( empty($title) || empty($_POST['comment_context_field_values'][$context_field_counter]) || empty($_POST['comment_context_field_questions'][$context_field_counter])){ continue; }
			
					
				//strip any trailing line-breaks
				$comment_values = preg_replace( "/\n$/" , '' , $_POST['comment_context_field_values'][$context_field_counter] );
				$comment_question = $_POST['comment_context_field_questions'][$context_field_counter];	
					
				$comment_context_array[] = array( 'title' => $title ,
												 'values' => explode( "\n" , str_replace("\r" , '' , $comment_values) ),
											   'question' => $comment_question );
									
				$context_field_counter++;
			}

			if ( update_post_meta($post_id, 'fun_with_in_context_comments', $comment_context_array) == false ){
				add_post_meta($post_id, 'fun_with_in_context_comments', $comment_context_array);
			}
			
		}
		
		/**
		 * calls the function to show the meta boxes
		 */
		function add_meta_boxes(){
			add_meta_box('fwicc_page_meta', 'Comment Context', array( &$this , 'comment_meta_box' ), 'page', 'advanced');
			add_meta_box('fwicc_post_meta', 'Comment Context', array( &$this , 'comment_meta_box' ), 'post', 'advanced');
		}


		/**
		 * comment_meta_box
		 * 
		 * Displays the contents of the meta box that appears on the page, and post screens.
		 * This is where the user chooses which questions to ask the commenter.
		 * 
		 * @return 
		 */ 
		function comment_meta_box(){
			global $post;
			
			if ( $meta = get_post_meta( $post->ID , 'fun_with_in_context_comments' , true) ) {
				$comment_fields = $meta;
			} else {
				$comment_fields = array( array( "question" => '' , "title" => '' , "values" => array()));	
			}
			
			if ( $meta = get_post_meta( $post->ID , 'fun_with_in_context_comments_globals' , true) ) {
				$global_comment_fields = $meta;
			} else {
				$global_comment_fields = array();	
			}
			
			// Use nonce for verification
			echo '<input type="hidden" name="fun_with_in_context_comments_add_context_fields" id="fun_with_in_context_comments_add_context_fields" value="' . wp_create_nonce( plugin_basename(__FILE__) ) . '" />';

			?>
			<p><?php _e('Select any global context fields you want to use on this post.' , 'fun_with_in_context_comments' ); ?></p>
			<?php
			
			foreach( $this->adminOptions['global_contexts'] as $context_id => $context ){ 
				//if the global field has been deleted then skip
				if ( $context == 'deleted' ){ continue; }
				
				$checked = ( in_array( $context_id , $global_comment_fields )  ) ? ' checked="checked" ' : '';
				?>
					<p><label><input type="checkbox"<?php echo $checked; ?> name="global_contexts[]" id="global_contexts[]" value="<?php echo $context_id; ?>"> <?php echo $context['title']; ?></label></p>	
				<?php
			}
			?>
			<p><?php _e('Use these fields to add drop down boxes to your comment fields.' , 'fun_with_in_context_comments' ); ?></p>
			<div id="context_fields">
			<?php $first_field = true; ?>
			<?php foreach( $comment_fields as $comment_field) {  ?>
			<div class="comment_context_fields">
				<p><label for="comment_context_field_titles[]"><?php _e('The question to ask commenters' , 'fun_with_in_context_comments' ); ?></label></p>
				<p><input type="text" class="context_input" name="comment_context_field_questions[]" size="25" id="comment_context_field_questions[]" value="<?php echo $comment_field['question']; ?>" /></p>
				<p><label for="comment_context_field_titles[]"><?php _e('The caption to accompany the additional information' , 'fun_with_in_context_comments' ); ?></label></p>
				<p><input type="text" class="context_input" name="comment_context_field_titles[]" size="25" id="comment_context_field_titles[]" value="<?php echo $comment_field['title']; ?>" /></p>
				<p><label for="comment_context_field_values[]"><?php _e('The values available for selection (one line per value)' , 'fun_with_in_context_comments' ); ?></label></p>
				<textarea cols='50' class="context_input" rows='5' name="comment_context_field_values[]" id="comment_context_field_values[]" ><?php echo implode("\n" , (array) $comment_field['values']); ?></textarea>
				<p class="remove_context_question"<?php if ( $first_field ) { echo ' style="display:none" '; $first_field = false; } ?>><a href="javascript:void(0)" onclick="jQuery(this).parents('div.comment_context_fields').remove();">Remove this question</a></p>
			</div>
			<?php } ?>
			<?php /* Using inline javascript to avoid additional extraneous scripts that might further impede load time in the admin panel */ ?>
			</div>
			<p><a href="javascript:void(0)" onclick="jQuery('div.comment_context_fields').clone().appendTo('#context_fields').children('p.remove_context_question').show().parents('div.comment_context_fields').find('.context_input').val('');">Add another question</a></p>
			<?php
		}
		
		/**
		 * output_context_fields
		 * 
		 * Echos the additional fields that will appear as part of the comments
		 * 
		 */
		function output_context_fields(){
			global $post;
			
			// Use nonce for verification
			echo '<input type="hidden" name="fun_with_in_context_comments_record_comment_context" id="fun_with_in_context_comments_record_comment_context" value="' . wp_create_nonce( plugin_basename(__FILE__) ) . '" />';
			
			//get global and local fields
			$global_fields = $this->adminOptions['global_contexts'];
			$selected_global_fields = get_post_meta( $post->ID , 'fun_with_in_context_comments_globals' , true);
			$local_fields = get_post_meta( $post->ID , 'fun_with_in_context_comments' , true);
			
			if ( is_array( $global_fields ) && is_array( $selected_global_fields ) ){
			
			//do global first
			foreach( $global_fields as $index => $comment_field ) {
				//if the global field has been deleted then skip
				if ( $comment_field == 'deleted' || !in_array( $index , $selected_global_fields ) ){ continue; }
				
				
				echo '<p><label for="fwicc_global_'.$index.'">';
				echo $comment_field['question'];
				echo '<br /><select name="fwicc_global_'.$index.'" id="fwicc_global_'.$index.'">';
				
				$comment_field_options_count = 0;
				
				//blank one to start
				echo '<option value="blank">'.__('Select if applicable' , "fun_with_in_context_comments" ).'</option>';
				
				foreach( $comment_field['values'] as $comment_field_value ){
					
					echo '<option value="'.$comment_field_options_count.'">';
					echo $comment_field_value;
					echo '</option>';
					$comment_field_options_count++;
					
				}
				
				echo '</select>';
				echo '</label></p>';
				
			}
			
			}
			
			if ( is_array( $local_fields ) ){
			
			//now to local
			foreach( $local_fields as $index => $comment_field ) {
					
				echo '<p><label for="fwicc_local_'.$index.'">';
				echo $comment_field['question'];
				echo '<br /><select name="fwicc_local_'.$index.'" id="fwicc_local_'.$index.'">';
				
				$comment_field_options_count = 0;
				
				//blank one to start
				echo '<option value="blank">'.__('Select if applicable' , "fun_with_in_context_comments" ).'</option>';
				
				foreach( $comment_field['values'] as $comment_field_value ){
					
					echo '<option value="'.$comment_field_options_count.'">';
					echo $comment_field_value;
					echo '</option>';
					$comment_field_options_count++;
					
				}
				
				echo '</select>';
				echo '</label></p>';
				
			}
			
			}
		
		}
		
		/**
		 * output_context_results
		 * 
		 * Echos the options the commenter chose.
		 * Must be used within the comment loop.
		 * Choices are displayed in the format: title separator value.
		 * 
		 * @param $before String[optional] Information to display before the values. Defaults to blank.
		 * @param $separator String[optional] The characters to display between the title and the value. Defaults to a colon.
		 * @param $between Object[optional] The characters to display between each selected value. Defaults to a comma.
		 * @param $none Object[optional] The text to display if there are no values (for older comments). Defaults to blank.
		 */
		function output_context_results( $before = '' , $separator = ' : ' , $between = ' , ' , $none = ''){
			global $comment, $wpdb;
			
			//get all values for this comment
			$values = $wpdb->get_results( sprintf("SELECT * FROM `%s` WHERE `comment_id` = %d" , $this->db_table_name , $comment->comment_ID)  );
			
			if ( count($values) > 0 ){
			
			$output_string = $before;
			
			foreach ( $values as $value ){
				
				if ( $output_string != $before ){
					$output_string .= $between;
				}
				
				$output_string .= $value->comment_field . $separator . $value->comment_value;
				
			}
			
			return $output_string;
			
			} else {
				return $none;
			}
			
		}
		

		
		/**
		* Retrieves the options from the database.
		* @return array
		*/
		function getAdminOptions() {
		$adminOptions = array("global_contexts" => array(), "settings" => array(
																				"before" => '',
																				"separator" => ' : ',
																				"between" => ', '),
																				"none" => '',
																				"fields_position" => 'above',
																				"template" => '<p style="font-size:10px;">%content%</p>',
																				"results_position" => 'above');
		$savedOptions = get_option($this->adminOptionsName);
		if (!empty($savedOptions)) {
			foreach ($savedOptions as $key => $option) {
				$adminOptions[$key] = $option;
			}
		}
		update_option($this->adminOptionsName, $adminOptions);
		return $adminOptions;
		}
		
		/**
		* Saves the admin options to the database.
		*/
		function saveAdminOptions(){
			update_option($this->adminOptionsName, $this->adminOptions);
		}
		
		function add_admin_pages(){
				add_submenu_page('edit-comments.php', "Global Contexts", "Global Contexts", 10, "Contexts", array(&$this,"output_sub_admin_page_0"));
		}
		
		/**
		* Outputs the HTML for the admin sub page.
		*/
		function output_sub_admin_page_0(){
			
			//edit nonce			
			if ( wp_verify_nonce( $_POST['_wpnonce'], 'fun_with_in_context_comments-add_context' )) {
			 
				// OK, we're authenticated: we need to find and save the data
				$comment_context_array = array();
									
				if ( isset( $_POST['comment_context_field_title'] ) ) {
						
					$title = $_POST['comment_context_field_title'];
						
					//strip any trailing line-breaks
					$comment_values = preg_replace( "/\n$/" , '' , $_POST['comment_context_field_values'] );
					$comment_question = $_POST['comment_context_field_question'];	
						
					$comment_context_array['title'] = $title;
					$comment_context_array['values'] = explode( "\n" , str_replace("\r" , '' , $comment_values) );
					$comment_context_array['question'] = $comment_question;
					
					if ( isset( $_POST['context_id'] ) && is_numeric($_POST['context_id']) ){
						$this->adminOptions['global_contexts'][$_POST['context_id']] = $comment_context_array;
					} else {
						$this->adminOptions['global_contexts'][] = $comment_context_array;	
					}
					
					$this->saveAdminOptions();
							
				}
			 }
			
			//delete nonce
			if ( wp_verify_nonce( $_POST['_wpnonce'], 'fun_with_in_context_comments-delete_contexts' )) {
				if ( isset( $_POST['delete_contexts'] ) ){
					foreach ( (array) $_POST['delete_contexts'] as $delete_index){
						//delete it instead of unsetting it so we can be sure the index isn't reused
						$this->adminOptions['global_contexts'][$delete_index] = 'deleted';
					}
					$this->saveAdminOptions();
	
				}
			}
			
			//get a context to edit
			if ( isset( $_GET['action'] ) && $_GET['action'] == 'edit' ){
				if ( isset( $_GET['id'] ) && is_numeric($_GET['id']) ){
					if ( isset( $this->adminOptions['global_contexts'][$_GET['id']] ) ){
						$context_to_edit = $this->adminOptions['global_contexts'][$_GET['id']];
						$context_to_edit['id'] = $_GET['id'];
					}
				}
			}
			
			//update settings
			if ( wp_verify_nonce( $_POST['_wpnonce'], 'fun_with_in_context_comments-update_settings' )) {
				if ( isset( $_POST['text_before'] ) ){
					$this->adminOptions['settings']['before'] = stripslashes($_POST['text_before']);
				}
				if ( isset( $_POST['text_between'] ) ){
					$this->adminOptions['settings']['between'] = stripslashes($_POST['text_between']);
				}
				if ( isset( $_POST['text_separator'] ) ){
					$this->adminOptions['settings']['separator'] = stripslashes($_POST['text_separator']);
				}
				if ( isset( $_POST['text_none'] ) ){
					$this->adminOptions['settings']['none'] = stripslashes($_POST['text_none']);
				}
				if ( isset( $_POST['context_template'] ) ){
					$this->adminOptions['settings']['template'] = stripslashes($_POST['context_template']);
				}
				if ( isset( $_POST['results_position'] ) ){
					$this->adminOptions['settings']['results_position'] = ( $_POST['results_position'] == 1 ) ? 'above' : 'below';
				}
				$this->saveAdminOptions();
			}
			
			
			?>
			<div class="wrap">
				<h2><?php _e('Global Context Settings' , 'fun_with_in_context_comments' ); ?></h2>
				<p><?php _e('Select the options for displaying the users selection' , 'fun_with_in_context_comments' ); ?></p>

				<form method="post" action="<?php bloginfo('wpurl'); ?>/wp-admin/edit-comments.php?page=Contexts" >
				<?php wp_nonce_field('fun_with_in_context_comments-update_settings'); ?>
				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row">
								<?php _e('Text to display before results' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<input id="text_before" type="text" size="40" value="<?php echo $this->adminOptions['settings']['before']; ?>" name="text_before"/>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<?php _e('Characters to display between results' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<input id="text_between" type="text" size="40" value="<?php echo $this->adminOptions['settings']['between']; ?>" name="text_between"/>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<?php _e('Text to separate the name from the result' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<input id="text_separator" type="text" size="40" value="<?php echo $this->adminOptions['settings']['separator']; ?>" name="text_separator"/>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<?php _e('HTML template to use: use %content% to show where the details of the users selections should go' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<input id="context_template" type="text" size="40" value="<?php echo htmlspecialchars( $this->adminOptions['settings']['template'] , ENT_QUOTES); ?>" name="context_template"/>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<?php _e('Text to display for no results' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<input id="text_none" type="text" size="40" value="<?php echo $this->adminOptions['settings']['none']; ?>" name="text_none"/>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<?php _e('Results Position' , 'fun_with_in_context_comments' ); ?>
							</th>
							<td>
								<select name="results_position">
									<option value="1">Before the comment text</option>
									<option value="2"<?php echo ($this->adminOptions['settings']['results_position'] == 'below') ? ' selected="selected"' : '' ; ?>>After the comment text</option>
								</select>
							</td>
						</tr>
					</tbody>
				</table>
				<p>
				<input value="Save Settings" name="saveSettings" class="button-secondary delete" type="submit">
				</p>
				</form>
			</div>
			<div class="wrap">
				<h2><?php _e('Global Contexts' , 'fun_with_in_context_comments' ); ?></h2>	
				<p><?php _e('Add contexts to use on any post' , 'fun_with_in_context_comments' ); ?></p>
				<form method="post" action="<?php bloginfo('wpurl'); ?>/wp-admin/edit-comments.php?page=Contexts" >
				<?php wp_nonce_field('fun_with_in_context_comments-delete_contexts'); ?>
				<div class="tablenav">
				<div class="alignleft">
					<input value="Delete" name="deleteit" class="button-secondary delete" type="submit">
				</div>
				</div>
				<br class="clear">
				<table class="widefat">
					<thead>
						<tr>
							<th class="check-column" scope="col"></th>
							<th scope="col">Title</th>
							<th scope="col">Question</th>
							<th scope="col">Options</th>
						</tr>
					</thead>
					<tbody>
						<?php $count = 1; ?>
						<?php foreach( $this->adminOptions['global_contexts'] as $context_id => $context ){ ?>
						<?php //if the global field has been deleted then skip
							  if ( $context == 'deleted' ){ continue; } 
						 ?>
						<tr>
							<th scope="row" class="check-column <?php if ($count == 1){echo 'alternate';} ?>"><input type="checkbox" valign="bottom" value="<?php echo $context_id; ?>" name="delete_contexts[]"/></th>
							
							<td class="<?php if ($count == 1){echo 'alternate';} ?>" valign="top"><strong><a href="edit-comments.php?page=Contexts&action=edit&id=<?php echo $context_id; ?>"><?php echo $context['title']; ?></a></strong></td>
							<td class="<?php if ($count == 1){echo 'alternate';} ?>" valign="top"><?php echo $context['question']; ?></td>
							<td class="<?php if ($count == 1){echo 'alternate'; $count = 0; } else { $count = 1; } ?>" valign="top"><?php foreach( $context['values'] as $c_value ){ echo $c_value . '<br />'; } ?></td>
						</tr>
						
						<?php } ?>
						
					</tbody>
					
				</table>
				</form>
				<form method="post" action="<?php bloginfo('wpurl'); ?>/wp-admin/edit-comments.php?page=Contexts">
				<h3><?php 
				if ( isset( $context_to_edit ) ) {
					_e('Update context question' , 'fun_with_in_context_comments' );
				} else {
					_e('Add context question' , 'fun_with_in_context_comments' );
				} ?></h3>
				<p><?php _e('Complete these fields to add a new global context' , 'fun_with_in_context_comments' ); ?></p>
				<div id="poststuff">
					<div id="postbody">
			
						<?php wp_nonce_field('fun_with_in_context_comments-add_context'); ?>
						<div id="comment_context_fields">
					
						<?php if ( isset( $context_to_edit ) ) { ?>
						<input type="hidden" name="context_id" id="context_id" value="<?php echo $context_to_edit['id']; ?>" />
						<?php } ?>
						
						<div class="stuffbox">
							<h3><?php _e('The question to ask commenters' , 'fun_with_in_context_comments' ); ?></h3>
							<div class="inside">
								<input type="text" class="context_input" name="comment_context_field_question" size="25" id="comment_context_field_question" value="<?php if ( isset ( $context_to_edit ) ) { echo $context_to_edit['question']; } ?>" />	
							</div>
						</div>
						<div class="stuffbox">
							<h3><?php _e('The caption to accompany the additional information' , 'fun_with_in_context_comments' ); ?></h3>
							<div class="inside">
								<input type="text" class="context_input" name="comment_context_field_title" size="25" id="comment_context_field_title" value="<?php if ( isset ( $context_to_edit ) ) { echo $context_to_edit['title']; } ?>" />	
							</div>
						</div>
						<div class="stuffbox">
							<h3><?php _e('The values available for selection (one line per value)' , 'fun_with_in_context_comments' ); ?></h3>
							<div class="inside">
								<textarea cols='50' class="context_input" rows='5' name="comment_context_field_values" id="comment_context_field_values" ><?php if ( isset ( $context_to_edit ) ) { echo implode( "\n" , $context_to_edit['values']); } ?></textarea>
							</div>
						</div>
					</div>	
					<p class="submit">
					<input class="button button-highlighted" name="save" value="<?php echo (isset( $context_to_edit ) ) ? 'save' : 'add';   ?>" tabindex="4" type="submit">
					</p>
					<?php if ( isset( $context_to_edit ) ) { ?>	
					<p><a href="<?php bloginfo('wpurl'); ?>/wp-admin/edit-comments.php?page=Contexts" title="cancel">Cancel</a></p>				
					<?php } ?>
					</div>
					</div>
				</form>
			</div>
		<?php
		} 
		
		/**
		* Creates or updates the database table, and adds a database table version number to the WordPress options.
		*/
		function install_on_activation() {
			global $wpdb;
			$plugin_db_version = "1.1";
			$installed_ver = get_option( "fun_with_in_context_comments_db_version" );
			//only run installation if not installed or if previous version installed
			if ($installed_ver === false || $installed_ver != $plugin_db_version) {
		
				//*****************************************************************************************
				// Create the sql - You will need to edit this to include the columns you need
				// Using the dbdelta function to allow the table to be updated if this is an update.
				// Read the limitations of the dbdelta function here: http://codex.wordpress.org/Creating_Tables_with_Plugins
				// remember to update the version number every time you want to make a change.
				//*****************************************************************************************
				$sql = "CREATE TABLE " . $this->db_table_name . " (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				comment_id mediumint(9) NOT NULL,
				comment_field VARCHAR(255) NOT NULL,
				comment_value VARCHAR(255) NOT NULL,
				UNIQUE KEY id (id)
				);";
			
				require_once(ABSPATH . "wp-admin/upgrade-functions.php");
				dbDelta($sql);
				//add a database version number for future upgrade purposes
				update_option("fun_with_in_context_comments_db_version", $plugin_db_version);
			}
		}

    }
}

//instantiate the class
if (class_exists('fw_in_context_comments')) {
	$fw_in_context_comments = new fw_in_context_comments();
}

/**
 * comment_context_fields
 * 
 * Wrapper around the function that echos the additional fields that will appear as part of the comments.
 * 
 */
function comment_context_fields(){
	global $fw_in_context_comments;
	
	$fw_in_context_comments->output_context_fields();
}


?>